# -*- coding: utf-8 -*-

from sklearn.ensemble import RandomForestClassifier
from sklearn.metrics import accuracy_score
from sklearn.neighbors import KNeighborsClassifier
from sklearn.neural_network import MLPClassifier
from sklearn.svm import SVC
from sklearn.tree import DecisionTreeClassifier

from DTWDistance import DTWDistance


class DefinedClassifiers:
    MAX_ITER = 1000
    ALPHA = 0.0001

    classifiers = {
        'KNN + Features (1)': lambda: KNeighborsClassifier(n_neighbors=1),
        'KNN + Features (3)': lambda: KNeighborsClassifier(n_neighbors=3),
        'KNN + Features (5)': lambda: KNeighborsClassifier(n_neighbors=5),
        'KNN + Features (7)': lambda: KNeighborsClassifier(n_neighbors=7),
        'KNN + Features (9)': lambda: KNeighborsClassifier(n_neighbors=9),
        'KNN + Features (11)': lambda: KNeighborsClassifier(n_neighbors=11),
        'DT': lambda: DecisionTreeClassifier(),
        'DT (5)': lambda: DecisionTreeClassifier(max_depth=5),
        'DT (10)': lambda: DecisionTreeClassifier(max_depth=10),
        'DT (15)': lambda: DecisionTreeClassifier(max_depth=15),
        'Linear SVM': lambda: SVC(kernel="linear", C=0.025),
        'SVM': lambda: SVC(gamma=2, C=1),
        'Random Forest (5)': lambda: RandomForestClassifier(max_depth=5, n_estimators=10, max_features=1),
        'MLP (relu; 100)': lambda: MLPClassifier(alpha=DefinedClassifiers.ALPHA, learning_rate='adaptive',
                                                 max_iter=DefinedClassifiers.MAX_ITER),
        'MLP (relu; 100x100)': lambda: MLPClassifier(hidden_layer_sizes=(100, 100), alpha=DefinedClassifiers.ALPHA,
                                                     learning_rate='adaptive', max_iter=DefinedClassifiers.MAX_ITER),
        'MLP (relu; 100x100x100)':
            lambda: MLPClassifier(hidden_layer_sizes=(100, 100, 100), alpha=DefinedClassifiers.ALPHA,
                                  learning_rate='adaptive',
                                  max_iter=DefinedClassifiers.MAX_ITER),
        'MLP (tanh; 100)':
            lambda: MLPClassifier(activation='tanh', alpha=DefinedClassifiers.ALPHA, learning_rate='adaptive',
                                  max_iter=DefinedClassifiers.MAX_ITER),
        'MLP (tanh; 100x100)':
            lambda: MLPClassifier(hidden_layer_sizes=(100, 100), activation='tanh', alpha=DefinedClassifiers.ALPHA,
                                  learning_rate='adaptive', max_iter=DefinedClassifiers.MAX_ITER),
        'MLP (tanh; 100x100x100)':
            lambda: MLPClassifier(hidden_layer_sizes=(100, 100, 100), activation='tanh', alpha=DefinedClassifiers.ALPHA,
                                  learning_rate='adaptive', max_iter=DefinedClassifiers.MAX_ITER),
    }

    knnDtwClassifiers = {
        'KNN+DTW (1)': lambda: KNeighborsClassifier(n_neighbors=1, algorithm='brute', metric=DTWDistance(), n_jobs=-2),
        'KNN+DTW (7)': lambda: KNeighborsClassifier(n_neighbors=7, algorithm='brute', metric=DTWDistance(), n_jobs=-2),
        'KNN+DTW (11)': lambda: KNeighborsClassifier(n_neighbors=11, algorithm='brute', metric=DTWDistance(), n_jobs=-2)
    }

    @staticmethod
    def cv_classifier(clf, cross_validation_set):
        scores = []

        for i, (X_train, X_valid, y_train, y_valid) in enumerate(cross_validation_set):
            print("CV iteration {}/{}".format(i + 1, len(cross_validation_set)))
            clf.fit(X_train, y_train)

            predicted_labels = clf.predict(X_valid)
            accuracy = accuracy_score(y_valid, predicted_labels)
            scores.append(accuracy)

        return clf, sum(scores) / len(scores)
