# -*- coding: utf-8 -*-

import numpy as np
from dtaidistance import dtw


class DTWDistance:

    def __init__(self):
        self._num_features = None

        try:
            a = np.zeros(5, dtype=np.double)
            b = np.zeros(5, dtype=np.double)
            dtw.distance_fast(a, b)
            self._dtw_distance = dtw.distance_fast
        except Exception:
            self._dtw_distance = dtw.distance

    @property
    def num_features(self):
        return self._num_features

    @num_features.setter
    def num_features(self, value):
        assert isinstance(value, int)
        self._num_features = value

    def __call__(self, a, b):
        assert self.num_features is not None
        assert a.size % self.num_features == 0
        assert b.size % self.num_features == 0

        a = a.reshape((-1, self.num_features))
        b = b.reshape((-1, self.num_features))

        distances = (self._dtw_distance(
            np.ascontiguousarray(a[:, i], dtype=np.double),
            np.ascontiguousarray(b[:, i], dtype=np.double)
        ) for i in range(self.num_features))

        return sum(distances)
